// this is an effect template. use it to start writing your own effects. 
 
 // ------------------------------------------------------------------------- 
 // PARAMETERS: 
 // ------------------------------------------------------------------------- 
 
 
 //transforms 
 float4x4 tW: WORLD; //the models world matrix 
 float4x4 tV: VIEW; //view matrix as set via Renderer (DX9) 
 float4x4 tP: PROJECTION; //projection matrix as set via Renderer (DX9) 
 float4x4 tWVP: WORLDVIEWPROJECTION; 
 
 //texture 
 texture Tex <string uiname="Texture";>; 
 sampler Samp = sampler_state //sampler for doing the texture-lookup 
 { 
 Texture = (Tex); //apply a texture to the sampler 
 MipFilter = LINEAR; //set the sampler states LINEAR 
 MinFilter = LINEAR; 
 MagFilter = LINEAR; 
 }; 
 //texture transformation marked with semantic TEXTUREMATRIX to achieve symmetric transformations 
 float4x4 tTex: TEXTUREMATRIX <string uiname="Texture Transform";>; 

 //kernel filter weights and the factor which is multiplicated 
 float factor <string uiname="Median Factor";> = 1.0;
  
 
 //pin for pixeldiameter
 float TexWidth <string uiname="Texture Width";> = 512;
 float TexHeight <string uiname="Texture Height";> = 512;
 
 //the data structure: "vertexshader to pixelshader" 
 //used as output data with the VS function 
 //and as input data with the PS function 
 struct vs2ps 
 { 
 float4 Pos : POSITION; 
 float4 TexCd : TEXCOORD0; 
 float2 Pixelsize : TEXCOORD1; 
 
 }; 
 
 
 // ------------------------------------------------------------------------- 
 // VERTEXSHADERS 
 // ------------------------------------------------------------------------- 
 
 vs2ps VS(float4 Pos : POSITION, float4 TexCd : TEXCOORD0) { 
 //inititalize all fields of output struct with 0 
 vs2ps Out = (vs2ps)0; 
 
 //transform position 
 Out.Pos = mul(Pos, tWVP); 
 
 //transform texturecoordinates 
 Out.TexCd = mul(TexCd, tTex); 
 
 //you can calculated things here aswell
 //we calculated here the "pixelsize" = 1/512
 //Out.Pixelsize.x = (1/TexWidth);
 //Out.Pixelsize.y = (1/TexHeight);

 //float2 size= (1/TexWidth,1/TexHeight);
 Out.Pixelsize = (1/TexWidth,1/TexHeight);

 return Out;
 } 
 
 // ------------------------------------------------------------------------- 
 // PIXELSHADERS: 
 // ------------------------------------------------------------------------- 


//Linear Blur filter 3x3
float4 PS_Blur_3_fast(vs2ps In): COLOR {

   float4 blurSamp = 0.0f;
   blurSamp += tex2D(Samp, In.TexCd + In.Pixelsize * float2(-1,-1)) * 1;
   blurSamp += tex2D(Samp, In.TexCd + In.Pixelsize * float2(0,-1)) * 2;
   blurSamp += tex2D(Samp, In.TexCd + In.Pixelsize * float2(1,-1)) * 1;

   blurSamp += tex2D(Samp, In.TexCd + In.Pixelsize * float2(-1,0)) * 2;
   blurSamp += tex2D(Samp, In.TexCd + In.Pixelsize * float2(0,0)) * 4;
   blurSamp += tex2D(Samp, In.TexCd + In.Pixelsize * float2(1,0)) * 2;

   blurSamp += tex2D(Samp, In.TexCd + In.Pixelsize * float2(-1,1)) * 1;
   blurSamp += tex2D(Samp, In.TexCd + In.Pixelsize * float2(0,1)) * 2;
   blurSamp += tex2D(Samp, In.TexCd + In.Pixelsize * float2(1,1)) * 1;

   blurSamp= blurSamp*0.0625;
   blurSamp = (0.222f * blurSamp.r + 0.707f * blurSamp.g + 0.071f * blurSamp.b);
   blurSamp.a=1.0f;
   return blurSamp;
}

//Linear Blur filter 3x3
float4 PS_Blur_3(vs2ps In): COLOR {


 // The  pixels 
 float2 neighbours[9] = { 
    -1, -1,
    0, -1,
    1, -1,
   -1,  0,
    0,  0,
    1,  0,
    -1,  1,
    0,  1,
    1,  1,
  };


 // fixed blur filter
 float blurWeights[9] = { 
   1,
   2,
   1,
   2,
   4,
   2,
   1,
   2,
   1,
  };

   float4 blurSamp = 0;
   for (int i = 0; i < 9; i++) {
      blurSamp += tex2D(Samp, In.TexCd + In.Pixelsize * neighbours[i]) * blurWeights[i];
    }

   return blurSamp*factor*0.0625;
}  


//Linear Blur filter 5x5
float4 PS_Blur_5(vs2ps In): COLOR {


 // The  pixels 
 float2 neighbours[21] = { 
    -1, -2,
    0, -2,
    1, -2,
    -2, -1,
    -1, -1,
    0, -1,
    1, -1,
    2, -1,
    -2, 0,
    -1, 0,
    0, 0,
    1, 0,
    2, 0,
    -2, 1,
    -1, 1,
    0, 1,
    1, 1,
    2, 1,
    -1, 2,
    0, 2,
    1, 2,
  };


 // fixed blur filter
 float blurWeights[21] = { 
   1,
   2,
   1,
   1,
   2,
   4,
   2,
   1,
   2,
   4,
   8,
   4,
   2,
   1,
   2,
   4,
   2,
   1,
   1,
   2,
   1,
  };

   float4 blurSamp = 0;
   for (int i = 0; i < 21; i++) {
      blurSamp += tex2D(Samp, In.TexCd + In.Pixelsize * neighbours[i]) * blurWeights[i];
    }

   return blurSamp*factor*0.0208333;
}  

 
//Median filter
float4 PS_medianFilter_3(vs2ps In): COLOR {


 // The  pixels 
 float2 neighbours[5] = { 
    0, -1,
   -1,  0,
    0,  0,
    1,  0,
    0,  1,
  };


 // fixed median filter
 float medWeights[5] = { 
   1,
   1,
   4,
   1,
   1,
  };

   float4 medSamp = 0;
   for (int i = 0; i < 5; i++) {
      medSamp += tex2D(Samp, In.TexCd + In.Pixelsize * neighbours[i]) * medWeights[i];
    }

   return medSamp*factor*0.13;
} 

 
//Median filter
float4 PS_medianFilter_5(vs2ps In): COLOR {


 // The  pixels 
 float2 neighbours[9] = { 
    0, -2,
    0, -1,
   -1,  0,
   -2,  0,
    0,  0,
    1,  0,
    2,  0,
    0,  1,
    0,  2,
  };


 // fixed median filter
 float medWeights[9] = { 
   1,
   1,
   1,
   1,
   8,
   1,
   1,
   1,
   1,
  };

   float4 medSamp = 0;
   for (int i = 0; i < 9; i++) {
      medSamp += tex2D(Samp, In.TexCd + In.Pixelsize * neighbours[i]) * medWeights[i];
    }

   return medSamp*factor*0.125;
} 

//Median filter
float4 PS_medianFilter_3Y(vs2ps In): COLOR {


 // The  pixels
 float2 neighbours[5] = {
    0, -2,
    0, -1,
    0,  0,
    0,  1,
    0, 2,
  };


 // fixed median filter
 float medWeights[5] = {
   1,
   1,
   6,
   1,
   1,
  };

   float4 medSamp = 0;
   for (int i = 0; i < 5; i++) {
      medSamp += tex2D(Samp, In.TexCd + In.Pixelsize * neighbours[i]) * medWeights[i];
    }

   return medSamp*factor*0.125;
}


// bypass 
float4 PS0(vs2ps In): COLOR { 
   float4 col = tex2D(Samp, float2 (In.TexCd.x, In.TexCd.y)); 
   return col; 
} 
 
 
 
 
 // ------------------------------------------------------------------------- 
 // TECHNIQUES: 
 // ------------------------------------------------------------------------- 

 technique Fast_Blur_3x3 {
   pass P0 {
     VertexShader = compile vs_1_1 VS();
     PixelShader = compile ps_2_0 PS_Blur_3_fast();
   }
 }

 technique Blur_3x3 {
   pass P0 { 
     VertexShader = compile vs_1_1 VS(); 
     PixelShader = compile ps_2_0 PS_Blur_3(); 
   } 
 } 

 technique Blur_5x5 { 
   pass P0 { 
     VertexShader = compile vs_1_1 VS(); 
     PixelShader = compile ps_2_0 PS_Blur_5(); 
   } 
 } 

 technique Median_3x3 { 
   pass P0 { 
     VertexShader = compile vs_1_1 VS(); 
     PixelShader = compile ps_2_0 PS_medianFilter_3(); 
   } 
 } 

 technique Median_5x5 { 
   pass P0 { 
     VertexShader = compile vs_1_1 VS(); 
     PixelShader = compile ps_2_0 PS_medianFilter_5(); 
   } 
 } 

  technique Median_3x3_Y {
   pass P0 {
     VertexShader = compile vs_1_1 VS();
     PixelShader = compile ps_2_0 PS_medianFilter_3Y();
   }
 }


 technique Bypass { 
   pass P0 { 
     VertexShader = compile vs_1_1 VS(); 
     PixelShader = compile ps_2_0 PS0(); 
   } 
 }
